@@ -14,6 +14,8 @@ XCOMM Define these now or down further below, see below for explanation.
 XCOMM #define CPPCompiler
 XCOMM #define XpmLibrary
 XCOMM #define XmLibrary
+XCOMM #define XawLibrary
+XCOMM #define Xaw3dLibrary
 XCOMM #define GLLibrary
 XCOMM #define TtfLibrary
 XCOMM #define GlttLibrary
@@ -111,6 +113,7 @@ XCOMM If its not with your X11 stuff you should set the following
 XCOMM  MOTIFHOME = /usr/local
 XCOMM  MOTIFHOME = /usr/dt
 
+XMDEF = -DHAVE_MOTIF
 XCOMM If you get an error "Cannot find Xm/PanedW.h" while compiling, set
 XCOMM XMINC to the directory Xm/PanedW.h is in.  Below is a guess.
 XCOMM  XMINC = -I$(MOTIFHOME)/include
@@ -127,6 +130,28 @@ XCOMM  EDITRESLIB = -lXmu
 
 XCOMM   *** END XM CONFIG SECTION ***
 
+XCOMM   *** BEGIN XAW CONFIG SECTION ***
+
+XCOMM Only options.c and xmlock.c uses Athena.
+XCOMM Use as fallback if you have or want Motif.
+XCOMM If you have XAW3D use that instead of Athena.
+XCOMM If your system has libXaw, remove the 'XCOMM  ' from the next line.
+XCOMM  #define XawLibrary
+
+#ifdef XawLibrary
+XMDEF = -DHAVE_ATHENA
+XCOMM  XMDEF = -DHAVE_XAW3D
+
+XMLIB = -lXaw
+XCOMM  XMLIB = -lXaw3d
+
+XCOMM Debugging with editres
+XCOMM  EDITRESDEF = -DUSE_XMU
+XCOMM  EDITRESLIB = -lXmu
+#endif
+
+XCOMM   *** END XAW CONFIG SECTION ***
+
 XCOMM   *** BEGIN MESA AND OPENGL CONFIG SECTION ***
 
 XCOMM Modes in modes/glx use this.
@@ -547,7 +572,7 @@ XCOMM  OPTDEF += -DSTAFF_FILE=\"/etc/xlock.staff\"
 XCOMM  OPTDEF += -DSTAFF_NETGROUP=\"/etc/xlock.netgroup\"
 
 DEFINES = -DDEF_FILESEARCHPATH=\"$(LIBDIR)/%T/%N%S\" \
-$(SYSTEMDEF) $(EDITRESDEF) $(SLEEPDEF) $(OPTDEF) $(RANDDEF) \
+$(SYSTEMDEF) $(XMDEF) $(EDITRESDEF) $(SLEEPDEF) $(OPTDEF) $(RANDDEF) \
 $(MODULEDEF) $(CHECKDEF) $(INTERACTIVEDEF) $(UNSTABLEDEF) $(PASSWDDEF) \
 $(XMINC) $(XAWINC) $(CPPDEF) $(XPMDEF) $(GLDEF) $(DTSAVERDEF) $(DPMSDEF) \
 $(SOUNDDEF) $(THREADDEF) $(THREADS_DEFINES) $(THREADS_CFLAGS) $(PASSWDINC) \
