@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* dclock --- floating digital clock or message */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)dclock.c	5.00 2000/11/01 xlockmore";
 
 #endif
@@ -89,22 +89,21 @@ static const char sccsid[] = "@(#)dclock.c	5.00 2000/11/01 xlockmore";
 
 #ifdef STANDALONE
 #define MODE_dclock
-#define PROGCLASS "Dclock"
-#define HACK_INIT init_dclock
-#define HACK_DRAW draw_dclock
-#define dclock_opts xlockmore_opts
 #define DEFAULTS "*delay: 10000 \n" \
- "*cycles: 10000 \n" \
- "*ncolors: 64 \n"
+	"*cycles: 10000 \n" \
+	"*ncolors: 64 \n" \
+
+# define free_dclock 0
+# define reshape_dclock 0
+# define dclock_handle_event 0
 #define BRIGHT_COLORS
 #define UNIFORM_COLORS
 #include "xlockmore.h"		/* in xscreensaver distribution */
-#include "mode.h"
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
+#include "iostuff.h"
 #include "util.h"
 #endif /* STANDALONE */
-#include "iostuff.h"
 
 #ifdef MODE_dclock
 
@@ -112,7 +111,7 @@ static const char sccsid[] = "@(#)dclock.c	5.00 2000/11/01 xlockmore";
 #define METRIC 1
 #endif
 
-#if METRIC
+#ifdef METRIC
 #define AREA_MIN 35.303144
 #define AREA_TIME_START 1184193000.0
 #else
@@ -271,7 +270,7 @@ static OptionStruct desc[] =
 	{(char *) "-/+mayan", (char *) "turn on/off 13 bak'tun (21 December 2012) countdown"},
 };
 
-ModeSpecOpt dclock_opts =
+ENTRYPOINT ModeSpecOpt dclock_opts =
 {sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
 
 #ifdef USE_MODULES
@@ -375,6 +374,10 @@ static dclockstruct *dclocks = (dclockstruct *) NULL;
 
 extern char *message;
 
+#ifdef STANDALONE
+char * message = NULL;
+#endif
+
 static time_t
 timeAtLastNewYear(time_t timeNow)
 {
@@ -391,7 +394,7 @@ timeAtLastNewYear(time_t timeNow)
 }
 
 #ifndef HAVE_SNPRINTF
-static double logbase;
+static double logbase = 0.0;
 #define BASE 10.0
 #define GROUP 3
 #endif
@@ -632,8 +635,11 @@ drawabinary(ModeInfo * mi, int startx, int starty, int dotcount, int digit)
 }
 
 static void
-free_dclock(Display *display, dclockstruct *dp)
+free_dclock_screen(Display *display, dclockstruct *dp)
 {
+	if (dp == NULL) {
+		return;
+	}
 	if (dp->fgGC != None) {
 		XFreeGC(display, dp->fgGC);
 		dp->fgGC = None;
@@ -646,7 +652,9 @@ free_dclock(Display *display, dclockstruct *dp)
 		XFreePixmap(display, dp->pixmap);
 		dp->pixmap = None;
 	}
- }
+	dp = NULL;
+}
+
 
 static void
 drawDclock(ModeInfo * mi)
@@ -807,7 +815,7 @@ drawDclock(ModeInfo * mi)
 
 		if (dp->pixmap)
 			MI_CLEARWINDOWCOLORMAPFAST(mi, gc, MI_BLACK_PIXEL(mi));
-		free_dclock(display, dp);
+		free_dclock_screen(display, dp);
 		dp->pixw = dp->text_width;
 		if (dp->led)
 			dp->pixh = dp->text_height;
@@ -815,7 +823,7 @@ drawDclock(ModeInfo * mi)
 			dp->pixh = (dp->lines + DELTA) * dp->text_height;
 		if ((dp->pixmap = XCreatePixmap(display, window,
 				dp->pixw, dp->pixh, 1)) == None) {
-			free_dclock(display, dp);
+			free_dclock_screen(display, dp);
 			dp->pixw = 0;
 			dp->pixh = 0;
 			return;
@@ -832,7 +840,7 @@ drawDclock(ModeInfo * mi)
 			| GCFont
 #endif
 			, &gcv)) == None) {
-			free_dclock(display, dp);
+			free_dclock_screen(display, dp);
 			dp->pixw = 0;
 			dp->pixh = 0;
 			return;
@@ -844,7 +852,7 @@ drawDclock(ModeInfo * mi)
 			| GCFont
 #endif
 			, &gcv)) == None) {
-			free_dclock(display, dp);
+			free_dclock_screen(display, dp);
 			dp->pixw = 0;
 			dp->pixh = 0;
 			return;
@@ -926,14 +934,14 @@ drawDclock(ModeInfo * mi)
 		1L);
 }
 
-void
+ENTRYPOINT void
 release_dclock(ModeInfo * mi)
 {
 	if (dclocks != NULL) {
 		int screen;
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
-			free_dclock(MI_DISPLAY(mi), &dclocks[screen]);
+			free_dclock_screen(MI_DISPLAY(mi), &dclocks[screen]);
 		free(dclocks);
 		dclocks = (dclockstruct *) NULL;
 	}
@@ -951,7 +959,7 @@ release_dclock(ModeInfo * mi)
 extern long timezone;
 #endif
 
-void
+ENTRYPOINT void
 init_dclock(ModeInfo * mi)
 {
 	Display *display = MI_DISPLAY(mi);
@@ -959,16 +967,13 @@ init_dclock(ModeInfo * mi)
 	time_t timeNow, timeLocal;
 	int i, j;
 
-	if (dclocks == NULL) {
-#ifndef HAVE_SNPRINTF
-		logbase = log(BASE);
-#endif
-		if ((dclocks = (dclockstruct *) calloc(MI_NUM_SCREENS(mi),
-				sizeof (dclockstruct))) == NULL)
-			return;
-	}
+	MI_INIT(mi, dclocks);
 	dp = &dclocks[MI_SCREEN(mi)];
 
+#ifndef HAVE_SNPRINTF
+	if (logbase == 0.0)
+		logbase = log(BASE);
+#endif
 	dp->width = MI_WIDTH(mi);
 	dp->height = MI_HEIGHT(mi);
 
@@ -1171,7 +1176,7 @@ init_dclock(ModeInfo * mi)
 			char *p, *p2, *p3, buf[BUFSIZ];
 			int height, w;
 
-			p = strncpy(buf, message, BUFSIZ);
+			p = strncpy(buf, message, BUFSIZ - 1);
 			while (strlen(p) > 0) {
 				p2 = p + strlen(p) - 1;
 				if (*p2 == '\n')
@@ -1317,7 +1322,7 @@ init_dclock(ModeInfo * mi)
 	XSetGraphicsExposures(display, MI_GC(mi), False);
 }
 
-void
+ENTRYPOINT void
 draw_dclock(ModeInfo * mi)
 {
 	dclockstruct *dp;
@@ -1336,10 +1341,14 @@ draw_dclock(ModeInfo * mi)
 	}
 }
 
-void
+#ifndef STANDALONE
+ENTRYPOINT void
 refresh_dclock(ModeInfo * mi)
 {
 	MI_CLEARWINDOW(mi);
 }
+#endif
+
+XSCREENSAVER_MODULE ("Dclock", dclock)
 
 #endif /* MODE_dclock */
