@@ -4,7 +4,7 @@
  *          spewing out messages
  */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)nose.c	5.00 2000/11/01 xlockmore";
 
 #endif
@@ -28,17 +28,17 @@ static const char sccsid[] = "@(#)nose.c	5.00 2000/11/01 xlockmore";
  *              xscreensaver but colors are random).
  * 27-Feb-1996: Added new ModeInfo arg to init and callback hooks.  Removed
  *              references to onepause, now uses MI_PAUSE(mi) interface.
- *              Ron Hitchens <ron@idiom.com>
+ *              Ron Hitchens <ron AT idiom.com>
  * 10-Oct-1995: A better way of handling fortunes from a file, thanks to
- *              Jouk Jansen <joukj@hrem.stm.tudelft.nl>.
+ *              Jouk Jansen <joukj AT hrem.nano.tudelft.nl>.
  * 21-Sep-1995: font option added, debugged for multiscreens
  * 12-Aug-1995: xlock version
- * 1992: xscreensaver version, noseguy (Jamie Zawinski <jwz@jwz.org>)
+ * 1992: xscreensaver version, noseguy (Jamie Zawinski <jwz AT jwz.org>)
  * 1990: X11 version, xnlock (Dan Heller <argv@sun.com>)
  */
 
 /*-
- * xscreensaver, Copyright (c) 1992 Jamie Zawinski <jwz@mcom.com>
+ * xscreensaver, Copyright (c) 1992 Jamie Zawinski <jwz AT jwz.org>
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -51,27 +51,27 @@ static const char sccsid[] = "@(#)nose.c	5.00 2000/11/01 xlockmore";
 
 #ifdef STANDALONE
 #define MODE_nose
-#define PROGCLASS "Nose"
-#define HACK_INIT init_nose
-#define HACK_DRAW draw_nose
-#define nose_opts xlockmore_opts
 #define DEFAULTS "*delay: 100000 \n" \
- "*ncolors: 64 \n" \
- "*font: \n" \
- "*text: \n" \
- "*filename: \n" \
- "*fortunefile: \n" \
- "*program: \n"
+	"*ncolors: 64 \n" \
+	"*font: fixed\n" \
+	"*text: \n" \
+	"*filename: \n" \
+	"*fortunefile: \n" \
+	"*program: \n" \
+
+# define free_nose 0
+# define reshape_nose 0
+# define nose_handle_event 0
 #define UNIFORM_COLORS
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
-#endif /* STANDALONE */
 #include "iostuff.h"
+#endif /* STANDALONE */
 
 #ifdef MODE_nose
 
-ModeSpecOpt nose_opts =
+ENTRYPOINT ModeSpecOpt nose_opts =
 {0, (XrmOptionDescRec *) NULL, 0, (argtype *) NULL, (OptionStruct *) NULL};
 
 #ifdef USE_MODULES
@@ -118,8 +118,8 @@ static int getFontHeight(XFontSet f)
 	if (f == None) {
 		return 8;
 	} else {
-		/*XmbTextExtents(mode_font, "My", strlen("My"), &ink, &log); */
-		XmbTextExtents(mode_font, "M", strlen("M"), &ink, &log);
+		XmbTextExtents(mode_font, "My", strlen("My"), &ink, &log);
+		/*XmbTextExtents(mode_font, "M", strlen("M"), &ink, &log);*/
 		return log.height;
 	}
 }
@@ -188,7 +188,7 @@ XSetStipple(d,g,p); XSetTSOrigin(d,g,x,y);\
 XFillRectangle(d,np,g,x,y,w,h)
 
 static void
-free_nose(Display *display, nosestruct *np)
+free_nose_screen(Display *display, nosestruct *np)
 {
 	int         pix;
 
@@ -493,7 +493,7 @@ walk(ModeInfo * mi, register int dir)
 	nosestruct *np = &noses[MI_SCREEN(mi)];
 	register int incr = 0;
 
-	if (dir & (LEFT | RIGHT)) {	/* left/right movement (mabye up/down too) */
+	if (dir & (LEFT | RIGHT)) {	/* left/right movement (maybe up/down too) */
 		np->up = -np->up;	/* bouncing effect (even if hit a wall) */
 		if (dir & LEFT) {
 			incr = X_INCR;
@@ -598,7 +598,7 @@ talk(ModeInfo * mi, Bool force_erase)
 	}
 	np->talking = True;
 	walk(mi, FRONT);
-	p = strncpy(buf, np->words, BUFSIZ);
+	p = strncpy(buf, np->words, BUFSIZ - 1);
 	while (strlen(p) > 0) {
 		p2 = p + strlen(p) - 1;
 		if (*p2 == '\n')
@@ -778,7 +778,7 @@ move(ModeInfo * mi)
 	np->state = MOVE;
 }
 
-void
+ENTRYPOINT void
 init_nose(ModeInfo * mi)
 {
 	Display    *display = MI_DISPLAY(mi);
@@ -787,11 +787,7 @@ init_nose(ModeInfo * mi)
 	XGCValues   gcv;
 	nosestruct *np;
 
-	if (noses == NULL) {
-		if ((noses = (nosestruct *) calloc(MI_NUM_SCREENS(mi),
-					       sizeof (nosestruct))) == NULL)
-			return;
-	}
+	MI_INIT(mi, noses);
 	np = &noses[MI_SCREEN(mi)];
 
 	np->width = MI_WIDTH(mi) + 2;
@@ -816,7 +812,7 @@ init_nose(ModeInfo * mi)
 	}
 	if (np->noseGC[0] == None)
 		if (!pickClothes(mi)) {
-			free_nose(display, np);
+			free_nose_screen(display, np);
 			return;
 		}
 	np->words = getWords(MI_SCREEN(mi), MI_NUM_SCREENS(mi));
@@ -833,7 +829,7 @@ init_nose(ModeInfo * mi)
 #endif
 				 GCForeground | GCBackground | GCGraphicsExposures,
 				 &gcv)) == None) {
-			free_nose(display, np);
+			free_nose_screen(display, np);
 			return;
 		}
 		gcv.foreground = MI_BLACK_PIXEL(mi);
@@ -844,7 +840,7 @@ init_nose(ModeInfo * mi)
 #endif
 				 GCForeground | GCBackground | GCGraphicsExposures,
 				 &gcv)) == None) {
-			free_nose(display, np);
+			free_nose_screen(display, np);
 			return;
 		}
 	}
@@ -867,7 +863,7 @@ init_nose(ModeInfo * mi)
 	XFlush(display);
 }
 
-void
+ENTRYPOINT void
 draw_nose(ModeInfo * mi)
 {
 	nosestruct *np;
@@ -893,14 +889,14 @@ draw_nose(ModeInfo * mi)
 	}
 }
 
-void
+ENTRYPOINT void
 release_nose(ModeInfo * mi)
 {
 	if (noses != NULL) {
 		int         screen;
 
 		for (screen = 0; screen < MI_NUM_SCREENS(mi); screen++)
-			free_nose(MI_DISPLAY(mi), &noses[screen]);
+			free_nose_screen(MI_DISPLAY(mi), &noses[screen]);
 		free(noses);
 		noses = (nosestruct *) NULL;
 	}
@@ -914,10 +910,14 @@ release_nose(ModeInfo * mi)
 	}
 }
 
-void
+#ifndef STANDALONE
+ENTRYPOINT void
 refresh_nose(ModeInfo * mi)
 {
 	MI_CLEARWINDOW(mi);
 }
+#endif
+
+XSCREENSAVER_MODULE ("Nose", nose)
 
 #endif /* MODE_nose */
