@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* random --- run random modes for a certain duration */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)random.c	5.00 2000/11/01 xlockmore";
 
 #endif
@@ -22,24 +22,25 @@ static const char sccsid[] = "@(#)random.c	5.00 2000/11/01 xlockmore";
  * Revision History:
  * 01-Nov-2000: Allocation checks
  * 10-May-1997: Made more compatible with xscreensaver :)
- * 18-Mar-1996: Ron Hitchens <ron@idiom.com>
+ * 18-Mar-1996: Ron Hitchens <ron AT idiom.com>
  *              Re-coded for the ModeInfo calling scheme.  Added the
  *              change hook.  Get ready for 3.8 release.
- * 23-Dec-1995: Ron Hitchens <ron@idiom.com>
+ * 23-Dec-1995: Ron Hitchens <ron AT idiom.com>
  *              Re-coded pickMode() to keep track of the modes, so
  *              that all modes are tried before there are any repeats.
  *              Also prevent a mode from being picked twice in a row
  *              (could happen as first pick after refreshing the list).
- * 04-Sep-1995: Written by Heath A. Kehoe <hakehoe@icaen.uiowa.edu>.
+ * 04-Sep-1995: Written by Heath A. Kehoe <hakehoe AT icaen.uiowa.edu>.
  *
  */
 
 #ifdef STANDALONE
-#define PROGCLASS "Random"
-#define HACK_INIT init_random
-#define HACK_DRAW draw_random
-#define random_opts xlockmore_opts
-#define DEFAULTS "*verbose: False \n"
+#define MODE_random
+#define DEFAULTS "*verbose: False \n" \
+
+# define free_random 0
+# define reshape_random 0
+# define random_handle_event 0
 #include "xlockmore.h"		/* in xscreensaver distribution */
 #else /* STANDALONE */
 #include "xlock.h"		/* in xlockmore distribution */
@@ -83,7 +84,7 @@ static OptionStruct desc[] =
 	{(char *) "-/+fullrandom", (char *) "turn on/off full random choice of mode-options"}
 };
 
-ModeSpecOpt random_opts =
+ENTRYPOINT ModeSpecOpt random_opts =
 {sizeof opts / sizeof opts[0], opts, sizeof vars / sizeof vars[0], vars, desc};
 
 #ifdef USE_MODULES
@@ -142,6 +143,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_atunnels
 	"atunnels",
 #endif
+#ifdef MODE_boxed
+	"boxed",
+#endif
 #ifdef MODE_bubble3d
 	"bubble3d",
 #endif
@@ -166,6 +170,9 @@ static char gl_modes[][MAXMODECHARS] =
 #ifdef MODE_lament
 	"lament",
 #endif
+#ifdef MODE_maze3d
+	"maze3d",
+#endif
 #ifdef MODE_moebius
 	"moebius",
 #endif
@@ -937,7 +944,8 @@ parsemodelist(ModeInfo * mi)
 	if (MI_IS_DEBUG(mi)) {
 		(void) fprintf(stderr, "%d mode%s: ", nmodes, ((nmodes == 1) ? "" : "s"));
 		for (i = 0; i < nmodes; i++)
-			(void) fprintf(stderr, "%d ", modes[i]);
+			(void) fprintf(stderr, "%s, ",
+				LockProcs[i].cmdline_arg);
 		(void) fprintf(stderr, "\n");
 	}
 	return True;
@@ -997,12 +1005,13 @@ setMode(ModeInfo * mi, int newmode)
 #endif
 }
 
-void
+ENTRYPOINT void
 init_random(ModeInfo * mi)
 {
 	randomstruct *rp;
 	int         i;
 
+	/* NO MI_INIT here*/
 	if (randoms == NULL) {
 		if ((randoms = (randomstruct *) calloc(MI_NUM_SCREENS(mi),
 					     sizeof (randomstruct))) == NULL)
@@ -1035,7 +1044,7 @@ init_random(ModeInfo * mi)
 	call_init_hook(&LockProcs[currentmode], mi);
 }
 
-void
+ENTRYPOINT void
 draw_random(ModeInfo * mi)
 {
 	int         scrn = MI_SCREEN(mi);
@@ -1076,7 +1085,16 @@ draw_random(ModeInfo * mi)
 	call_callback_hook(&LockProcs[currentmode], mi);
 }
 
-void
+ENTRYPOINT void
+release_random(ModeInfo * mi)
+{
+	if (previousmode >= 0 && previousmode != currentmode)
+		call_release_hook(&LockProcs[previousmode], mi);
+	previousmode = currentmode;
+}
+
+#ifndef STANDALONE
+ENTRYPOINT void
 refresh_random(ModeInfo * mi)
 {
 	if (currentmode < 0)
@@ -1084,7 +1102,7 @@ refresh_random(ModeInfo * mi)
 	call_refresh_hook(&LockProcs[currentmode], mi);
 }
 
-void
+ENTRYPOINT void
 change_random(ModeInfo * mi)
 {
 	if (currentmode < 0)
@@ -1093,11 +1111,6 @@ change_random(ModeInfo * mi)
 		change_now = True;	/* force a change on next draw callback */
 	draw_random(mi);
 }
+#endif
 
-void
-release_random(ModeInfo * mi)
-{
-	if (previousmode >= 0 && previousmode != currentmode)
-		call_release_hook(&LockProcs[previousmode], mi);
-	previousmode = currentmode;
-}
+XSCREENSAVER_MODULE ("Random", random)
