@@ -156,7 +156,7 @@ BOOL WINAPI ScreenSaverConfigureDialog(hDlg, message, wParam, lParam)
                                    0,0,0,KEY_ALL_ACCESS,0,&skey,0)
                     !=ERROR_SUCCESS) return TRUE;
                 sprintf(buf, "%d", enabled[m]);
-                RegSetValueEx(skey,"Enabled",0,REG_SZ,buf,1);
+                RegSetValueEx(skey,"Enabled",0,REG_SZ,(LPBYTE)buf,1);
                 RegCloseKey(skey);
               }
             }
@@ -170,7 +170,7 @@ BOOL WINAPI ScreenSaverConfigureDialog(hDlg, message, wParam, lParam)
     return FALSE;
 }
 
-LONG WINAPI ScreenSaverProc(HWND myhwnd, UINT msg, WPARAM wParam, LPARAM lParam)
+LRESULT WINAPI ScreenSaverProc(HWND myhwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
   static UINT         uTimer;   // timer identifier
   static UINT         uHasInit = 0;
