@@ -1,4 +1,4 @@
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)mode.c	5.12 04/03/09 xlockmore";
 
 #endif
@@ -10,13 +10,16 @@ static const char sccsid[] = "@(#)mode.c	5.12 04/03/09 xlockmore";
  * Revision History:
  * 01-Apr-97: Fixed memory leak in XStringListToTextProperty.
  * 23-Feb-96: Extensive revision to implement new mode hooks and stuff
- *		Ron Hitchens <ron@idiom.com>
+ *		Ron Hitchens <ron AT idiom.com>
  * 04-Sep-95: Moved over from mode.h (previously resource.h) with new
- *            "&*_opts" by Heath A. Kehoe <hakehoe@icaen.uiowa.edu>.
+ *            "&*_opts" by Heath A. Kehoe <hakehoe AT icaen.uiowa.edu>.
  *
  */
 
 #include "xlock.h"
+#ifdef USE_GL
+# include "visgl.h"
+#endif
 
 
 /* -------------------------------------------------------------------- */
@@ -35,49 +38,49 @@ LockStruct  LockProcs[] =
 {
 #ifdef MODE_anemone
 	{(char *) "anemone", init_anemone, draw_anemone, release_anemone,
-	 init_apollonian, init_anemone, (ModeHook *) NULL, &anemone_opts,
+	 (ModeHook *)  NULL, init_anemone, free_anemone, &anemone_opts,
 	 50000, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows wiggling tentacles", 0, NULL},
 #endif
 #ifdef MODE_ant
 	{(char *) "ant", init_ant, draw_ant, release_ant,
-	 refresh_ant, init_ant, (ModeHook *) NULL, &ant_opts,
+	 refresh_ant, init_ant, free_ant, &ant_opts,
 	 1000, -3, 40000, -12, 64, 1.0, (char *) "",
 	 (char *) "Shows Langton's and Turk's generalized ants", 0, NULL},
 #endif
 #ifdef MODE_ant3d
 	{(char *) "ant3d", init_ant3d, draw_ant3d, release_ant3d,
-	 refresh_ant3d, init_ant3d, (ModeHook *) NULL, &ant3d_opts,
+	 refresh_ant3d, init_ant3d, free_ant3d, &ant3d_opts,
 	 5000, -3, 10000, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows 3D ants", 0, NULL},
 #endif
 #ifdef MODE_apollonian
 	{(char *) "apollonian", init_apollonian, draw_apollonian, release_apollonian,
-	 init_apollonian, init_apollonian, (ModeHook *) NULL, &apollonian_opts,
+	 init_apollonian, init_apollonian, free_apollonian, &apollonian_opts,
 	 1000000, 64, 20, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows Apollonian circles", 0, NULL},
 #endif
 #ifdef MODE_atlantis
 	{(char *) "atlantis", init_atlantis, draw_atlantis, release_atlantis,
-	 refresh_atlantis, change_atlantis, (ModeHook *) NULL, &atlantis_opts,
+	 (ModeHook *) NULL, change_atlantis, (ModeHook *) NULL, &atlantis_opts,
 	 25000, 4, 100, 6000, 64, 1.0, (char *) "",
 	 (char *) "Shows moving sharks/whales/dolphin", 0, NULL},
 #endif
 #ifdef MODE_atunnels
 	{(char *) "atunnels", init_atunnels, draw_atunnels, release_atunnels,
-	 refresh_atunnels, change_atunnels, (ModeHook *) NULL, &atunnels_opts,
+	 draw_atunnels, change_atunnels, (ModeHook *) NULL, &atunnels_opts,
 	 25000, 1, 1, 0, 64, 1.0, (char *) "",
 	 (char *) "Shows an OpenGL advanced tunnel screensaver", 0, NULL},
 #endif
 #ifdef MODE_ball
 	{(char *) "ball", init_ball, draw_ball, release_ball,
-	 refresh_ball, init_ball, (ModeHook *) NULL, &ball_opts,
+	 refresh_ball, init_ball, free_ball, &ball_opts,
 	 10000, 10, 20, -100, 64, 1.0, (char *) "",
 	 (char *) "Shows bouncing balls", 0, NULL},
 #endif
 #ifdef MODE_bat
 	{(char *) "bat", init_bat, draw_bat, release_bat,
-	 refresh_bat, init_bat, (ModeHook *) NULL, &bat_opts,
+	 refresh_bat, init_bat, free_bat, &bat_opts,
 	 100000, -8, 1, 0, 64, 1.0, (char *) "",
 	 (char *) "Shows bouncing flying bats", 0, NULL},
 #endif
@@ -89,7 +92,7 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef MODE_blot
 	{(char *) "blot", init_blot, draw_blot, release_blot,
-	 refresh_blot, init_blot, (ModeHook *) NULL, &blot_opts,
+	 refresh_blot, init_blot, free_blot, &blot_opts,
 	 200000, 6, 30, 1, 64, 0.3, (char *) "",
 	 (char *) "Shows Rorschach's ink blot test", 0, NULL},
 #endif
@@ -105,6 +108,12 @@ LockStruct  LockProcs[] =
 	 5000, -10, 1, 0, 64, 1.0, (char *) "",
 	 (char *) "Shows bouncing footballs", 0, NULL},
 #endif
+#ifdef MODE_boxed
+	{(char *) "boxed", init_boxed, draw_boxed, release_boxed,
+	draw_boxed, init_boxed, (ModeHook *) NULL, &boxed_opts,
+	1000, 1, 2, 1, 64, 1.0, (char *) "",
+	(char *) "Shows GL's boxed balls", 0, NULL},
+#endif
 #ifdef MODE_braid
 	{(char *) "braid", init_braid, draw_braid, release_braid,
 	 refresh_braid, init_braid, (ModeHook *) NULL, &braid_opts,
@@ -113,13 +122,13 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef MODE_bubble
 	{(char *) "bubble", init_bubble, draw_bubble, release_bubble,
-	 refresh_bubble, init_bubble, (ModeHook *) NULL, &bubble_opts,
+	 (ModeHook *) NULL, init_bubble, free_bubble, &bubble_opts,
 	 100000, 25, 1, 100, 64, 0.3, (char *) "",
 	 (char *) "Shows popping bubbles", 0, NULL},
 #endif
 #ifdef MODE_bubble3d
 	{(char *) "bubble3d", init_bubble3d, draw_bubble3d, release_bubble3d,
-	 draw_bubble3d, change_bubble3d, (ModeHook *) NULL, &bubble3d_opts,
+	 draw_bubble3d, (ModeHook *) NULL, (ModeHook *) NULL, &bubble3d_opts,
 	 20000, 1, 2, 1, 64, 1.0, (char *) "",
 	 (char *) "Richard Jones's GL bubbles", 0, NULL},
 #endif
@@ -185,7 +194,7 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef MODE_deluxe
 	{(char *) "deluxe", init_deluxe, draw_deluxe, release_deluxe,
-	 refresh_deluxe, init_deluxe, (ModeHook *) NULL, &deluxe_opts,
+	 (ModeHook *) NULL, init_deluxe, (ModeHook *) NULL, &deluxe_opts,
 	 5000, 5, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows pulsing sequence of stars, circles, and lines", 0, NULL},
 #endif
@@ -369,18 +378,18 @@ LockStruct  LockProcs[] =
 	 10000, 1000, 20, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows the Julia set", 0, NULL},
 #endif
-#ifdef MODE_kaleid2
-	{"kaleid2", init_kaleid2, draw_kaleid2, release_kaleid2,
-	     refresh_kaleid2, init_kaleid2, NULL, &kaleid2_opts,
-	     20000, 1, 700, 1, 64, 1.0, "",
-	     "Shows a kaleidoscope", 0, NULL},
-#endif
 #ifdef MODE_kaleid
 	{(char *) "kaleid", init_kaleid, draw_kaleid, release_kaleid,
 	 refresh_kaleid, init_kaleid, (ModeHook *) NULL, &kaleid_opts,
 	 80000, 4, 40, -9, 64, 0.6, (char *) "",
 	 (char *) "Shows a kaleidoscope", 0, NULL},
 #endif
+#ifdef MODE_kaleid2
+ 	{"kaleid2", init_kaleid2, draw_kaleid2, release_kaleid2,
+	 refresh_kaleid2, init_kaleid2, NULL, &kaleid2_opts,
+	 20000, 1, 700, 1, 64, 1.0, "",
+	 (char *) "Shows a kaleidoscope", 0, NULL},
+#endif
 #ifdef MODE_kumppa
 	{(char *) "kumppa", init_kumppa, draw_kumppa, release_kumppa,
 	 init_kumppa, init_kumppa, (ModeHook *) NULL, &kumppa_opts,
@@ -395,31 +404,31 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef MODE_laser
 	{(char *) "laser", init_laser, draw_laser, release_laser,
-	 refresh_laser, init_laser, (ModeHook *) NULL, &laser_opts,
+	 refresh_laser, init_laser, free_laser, &laser_opts,
 	 20000, -10, 200, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows spinning lasers", 0, NULL},
 #endif
 #ifdef MODE_life
 	{(char *) "life", init_life, draw_life, release_life,
-	 refresh_life, change_life, (ModeHook *) NULL, &life_opts,
+	 refresh_life, change_life, free_life, &life_opts,
 	 750000, 40, 140, 0, 64, 1.0, (char *) "",
 	 (char *) "Shows Conway's game of Life", 0, NULL},
 #endif
 #ifdef MODE_life1d
 	{(char *) "life1d", init_life1d, draw_life1d, release_life1d,
-	 refresh_life1d, init_life1d, (ModeHook *) NULL, &life1d_opts,
+	 refresh_life1d, init_life1d, free_life1d, &life1d_opts,
 	 100000, -5, 1200, -15, 64, 1.0, (char *) "",
 	 (char *) "Shows Wolfram's game of 1D Life", 0, NULL},
 #endif
 #ifdef MODE_life3d
 	{(char *) "life3d", init_life3d, draw_life3d, release_life3d,
-	 refresh_life3d, change_life3d, (ModeHook *) NULL, &life3d_opts,
+	 refresh_life3d, change_life3d, free_life3d, &life3d_opts,
 	 1000000, 35, 85, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows Bays' game of 3D Life", 0, NULL},
 #endif
 #ifdef MODE_lightning
 	{(char *) "lightning", init_lightning, draw_lightning, release_lightning,
-	 refresh_lightning, init_lightning, (ModeHook *) NULL, &lightning_opts,
+	 (ModeHook *) NULL, init_lightning, (ModeHook *) NULL, &lightning_opts,
 	 10000, 1, 1, 1, 64, 0.6, (char *) "",
 	 (char *) "Shows Keith's fractal lightning bolts", 0, NULL},
 #endif
@@ -471,6 +480,12 @@ LockStruct  LockProcs[] =
 	 1000, 1, 3000, -40, 64, 1.0, (char *) "",
 	 (char *) "Shows a random maze and a depth first search solution", 0, NULL},
 #endif
+#ifdef MODE_maze3d
+	{(char *) "maze3d", init_maze3d, draw_maze3d, release_maze3d,
+	 draw_maze3d, init_maze3d, (ModeHook *) NULL, &maze3d_opts,
+	 20000, 1, 2, 1, 64, 1.0, (char *) "",
+	 (char *) "Shows a 3D maze", 0, NULL},
+#endif
 #ifdef MODE_moebius
 	{(char *) "moebius", init_moebius, draw_moebius, release_moebius,
 	 draw_moebius, change_moebius, (ModeHook *) NULL, &moebius_opts,
@@ -574,7 +589,7 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef MODE_qix
 	{(char *) "qix", init_qix, draw_qix, release_qix,
-	 refresh_qix, init_qix, (ModeHook *) NULL, &qix_opts,
+	 refresh_qix, init_qix, free_qix, &qix_opts,
 	 30000, -5, 32, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows spinning lines a la Qix(tm)", 0, NULL},
 #endif
@@ -652,7 +667,7 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef MODE_space
 	{(char *) "space", init_space, draw_space, release_space,
-	 refresh_space, init_space, (ModeHook *) NULL, &space_opts,
+	 (ModeHook *) NULL, init_space, (ModeHook *) NULL, &space_opts,
 	 10000, 100, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows a journey into deep space", 0, NULL},
 #endif
@@ -676,7 +691,7 @@ LockStruct  LockProcs[] =
 #endif
 #ifdef MODE_sproingies
 	{(char *) "sproingies", init_sproingies, draw_sproingies, release_sproingies,
-	 refresh_sproingies, init_sproingies, (ModeHook *) NULL, &sproingies_opts,
+	 (ModeHook *) NULL, init_sproingies, (ModeHook *) NULL, &sproingies_opts,
 	 80000, 5, 0, 0, 64, 1.0, (char *) "",
   (char *) "Shows Sproingies!  Nontoxic.  Safe for pets and small children", 0, NULL},
 #endif
@@ -701,12 +716,12 @@ LockStruct  LockProcs[] =
 #ifdef MODE_strange
 	{(char *) "strange", init_strange, draw_strange, release_strange,
 	 init_strange, init_strange, (ModeHook *) NULL, &strange_opts,
-	 1000, 1, 1, 1, 64, 1.0, (char *) "",
+	 10000, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows strange attractors", 0, NULL},
 #endif
 #ifdef MODE_superquadrics
 	{(char *) "superquadrics", init_superquadrics, draw_superquadrics, release_superquadrics,
-	 refresh_superquadrics, init_superquadrics, (ModeHook *) NULL, &superquadrics_opts,
+	 (ModeHook *) NULL, init_superquadrics, (ModeHook *) NULL, &superquadrics_opts,
 	 40000, 25, 40, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows 3D mathematical shapes", 0, NULL},
 #endif
@@ -833,14 +848,14 @@ LockStruct  LockProcs[] =
 
 /* SPECIAL MODES */
 #ifndef WIN32
-	{(char *) "blank", init_blank, draw_blank, release_blank,
-	 refresh_blank, init_blank, (ModeHook *) NULL, &blank_opts,
+	{(char *) "blank", init_blank, (ModeHook *) NULL, (ModeHook *) NULL,
+	 (ModeHook *) NULL, init_blank, (ModeHook *) NULL, &blank_opts,
 	 3000000, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows nothing but a black screen", 0, NULL},
 #endif /* !WIN32 */
 #ifdef MODE_run
-	{(char *) "run", init_run, draw_run, release_run,
-	 refresh_run, init_run, (ModeHook *) NULL, &run_opts,
+	{(char *) "run", init_run, (ModeHook *) NULL, release_run,
+	 (ModeHook *) NULL, init_run, free_run, &run_opts,
 	 3000000, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows another xprogram", 0, NULL},
 #endif
@@ -974,7 +989,7 @@ ExpandPath(char *path)
 
 /*-
  * Loads screensaver modules in the directories in path.  path is a
- * colon separated list of direcories to search in.  LoadModules
+ * colon separated list of directories to search in.  LoadModules
  * searches for files ending in .xlk to load as modules.
  */
 
@@ -1105,13 +1120,13 @@ LoadModules(char *path)
 					}
 				} else
 					newstack->lock->change_hook = NULL;
-				if (desc->unused_name != NULL) {
+				/*if (desc->unused_name != NULL) {
 					newstack->lock->unused_hook = (ModeHook *) dlsym(mp, desc->unused_name);
 					if (newstack->lock->unused_hook == NULL) {
 						nextone;
 					}
 				} else
-					newstack->lock->unused_hook = NULL;
+					newstack->lock->unused_hook = NULL;*/
 				newstack->lock->msopt = desc->msopt;
 				newstack->lock->def_delay = desc->def_delay;
 				newstack->lock->def_count = desc->def_count;
@@ -1236,9 +1251,28 @@ call_release_hook(LockStruct * ls, ModeInfo * mi)
 		return;
 	}
 	MI_LOCKSTRUCT(mi) = ls;
+
+	if (ls->free_hook != NULL) {
+		int old_screen = MI_SCREEN(mi);
+		for (MI_SCREEN(mi) = 0; MI_SCREEN(mi) != MI_NUM_SCREENS(mi); ++MI_SCREEN(mi))
+			ls->free_hook (mi);
+		MI_SCREEN(mi) = old_screen;
+	}
+
 	if (ls->release_hook != NULL) {
 		ls->release_hook(mi);
 	}
+
+	if (ls->state_array) { /* MI_INIT modes only. */
+		free(*ls->state_array);
+		*ls->state_array = NULL;
+		ls->state_array = NULL;
+
+#ifdef USE_GL
+		FreeAllGL(mi); /* Should be a safe no-op for Xlib modes. */
+#endif
+	}
+
 	ls->flags &= ~(LS_FLAG_INITED);
 
 	last_initted_mode = (LockStruct *) NULL;
@@ -1255,6 +1289,25 @@ release_last_mode(ModeInfo * mi)
 	last_initted_mode = (LockStruct *) NULL;
 }
 
+void
+mi_init(ModeInfo *mi, size_t state_size, void **state_array)
+{
+	LockStruct *ls = MI_LOCKSTRUCT(mi);
+	ls->state_array = state_array;
+
+	if (!*state_array) {
+		*state_array = calloc (MI_NUM_SCREENS(mi), state_size);
+
+		if (!*state_array) {
+			return;
+		}
+	}
+
+	if (ls->free_hook)
+		ls->free_hook (mi);
+	memset ((char *)(*ls->state_array) + MI_SCREEN(mi) * state_size, 0, state_size);
+}
+
 /* -------------------------------------------------------------------- */
 
 /*-
@@ -1310,7 +1363,8 @@ call_callback_hook(LockStruct * ls, ModeInfo * mi)
 	}
 	MI_LOCKSTRUCT(mi) = ls;
 
-	ls->callback_hook(mi);
+	if (ls->callback_hook != NULL)
+		ls->callback_hook(mi);
 
 	MI_SET_FLAG_STATE(mi, WI_FLAG_JUST_INITTED, False);
 }
@@ -1384,3 +1438,24 @@ call_change_hook(LockStruct * ls, ModeInfo * mi)
 		ls->change_hook(mi);
 	}
 }
+
+/*-
+ *    Future?
+ */
+
+/*void
+call_unused_hook(LockStruct * ls, ModeInfo * mi)
+{
+	if (ls == NULL) {
+		if (default_mode == NULL) {
+			return;
+		} else {
+			ls = default_mode;
+		}
+	}
+	MI_LOCKSTRUCT(mi) = ls;
+
+	if (ls->unused_hook != NULL) {
+		ls->unused_hook(mi);
+	}
+}*/
